
#pragma once

#include "irc_entity.h"
#include "irc_connection_queues.h"

class IRCMessage;
class IRCText;

class IRCConnection : public CL_KeepAliveObject
{
public:
	IRCConnection();
	~IRCConnection();

	void connect(const CL_SocketName &server);
	void disconnect_graceful();
	void disconnect_abortive();

	void send_command(const IRCRawString &command, const std::vector<IRCRawString> params);

	void send_pass(const IRCRawString &password);
	void send_nick(const IRCNick &nickname);
	void send_user(const IRCRawString &username, const IRCRawString &hostname, const IRCRawString &servername, const IRCRawString &realname);
	void send_oper(const IRCRawString &user, const IRCRawString &password);
	void send_quit(const IRCText &message);
	void send_join(const IRCChannel &channel, const IRCRawString &password = IRCRawString());
	void send_part(const IRCChannel &channel);
	void send_channel_mode(const IRCChannel &channel, const IRCRawString &mode_command, const std::vector<IRCRawString> &mode_params);
	void send_nick_mode(const IRCNick &nickname, const IRCRawString &mode_command);
	void send_topic(const IRCChannel &channel, const IRCText &topic);
	void send_names(const IRCChannel &channel); // Request (RPL_NAMREPLY, RPL_ENDOFNAMES) for a channel
	void send_list(const IRCChannel &channel); // Request (RPL_LISTSTART, RPL_LIST, RPL_LISTEND) for a channel
	void send_invite(const IRCNick &nickname, const IRCChannel &channel);
	void send_kick(const IRCChannel &channel, const IRCNick &nickname, const IRCText &comment);

	void send_privmsg(const IRCEntity &receiver, const IRCText &text);
	void send_notice(const IRCEntity &receiver, const IRCText &text);

	void send_who(const IRCEntity &name_search, bool only_operators);
	void send_whois(const IRCEntity &name_search);
	void send_whowas(const IRCEntity &name);

	void send_kill(const IRCEntity &target, const IRCText &comment);

	void send_ping(const IRCServer &server, const IRCServer &server2 = IRCServer());
	void send_pong(const IRCRawString &daemon1, const IRCRawString &daemon2 = IRCRawString());

	// on_message_received(const IRCMessage &message)
	CL_Callback_v1<const IRCMessage &> &func_message_received();

	// on_disconnected(const CL_String &reason)
	CL_Callback_v1<const CL_String &> &func_disconnected();

	void process();
	CL_Event get_wakeup_event();

private:
	void worker_main();
	bool read_connection_data(CL_TCPConnection &connection, IRCRawString &read_line);
	bool write_connection_data(IRCRawString &write_line, IRCRawString::size_type &write_pos, CL_TCPConnection &connection);

	CL_SocketName server;
	CL_Thread thread_worker;
	CL_Event stop_event;
	CL_InterlockedVariable shutdown_graceful;
	IRCConnectionQueues queues;

	CL_Callback_v1<const IRCMessage &> cb_message_received;
	CL_Callback_v1<const CL_String &> cb_disconnected;
};
